package kernel.turtle;

import kernel.log.Log;
import cc.OS;
import kernel.ps.ProcessManager.PID;

/**
	Make sure only one process can access the turtle at a time.
**/
class TurtleMutex {
	private static var claimedPid:PID = -1;
	public static var threadFunc:() -> Void;

	@:allow(kernel.ps.ProcessHandle)
	private static function claim(pid:PID):Bool {
		if (claimedPid == -1) {
			claimedPid = pid;
			return true;
		}
		return false;
	}

	@:allow(kernel.ps.ProcessHandle)
	private static function release(pid:PID) {
		if (claimedPid == pid) {
			claimedPid = -1;
			stopTurtleThread();
		}
	}

	@:allow(kernel.KernelEvents)
	private static function runThreadFunc() {
		if (threadFunc != null) {
			try {
				threadFunc();
			} catch (e) {
				Log.error("Error in tthread: " + e);
			}
			threadFunc = null;
		}
	}

	public static function runInTThread(func:() -> Void) {
		threadFunc = func;
		OS.queueEvent("tthread");
	}

	public static function stopTurtleThread() {
		threadFunc = null;
		KernelEvents.startTurtleCoroutine();
	}
}
